import bpy, csv
from math import floor

realSTART = 25200
realEND = 28800
frameDURATION = 2500
frameTRESHOLD = 15

def animate(curve):
    curve.data.bevel_factor_end = 0.0
    if curve.data.animation_data == None or curve.data.animation_data.action == None:
        curve.data.keyframe_insert("bevel_factor_start", frame=1)
        curve.data.keyframe_insert("bevel_factor_end", frame=1)
    else:
        bstart = [fc for fc in curve.data.animation_data.action.fcurves if fc.data_path == "bevel_factor_start"]
        bend   = [fc for fc in curve.data.animation_data.action.fcurves if fc.data_path == "bevel_factor_end"]
        if len(bstart) == 0:
            curve.data.keyframe_insert("bevel_factor_start", frame=1)
        if len(bend) == 0:
            curve.data.keyframe_insert("bevel_factor_end", frame=1)
    
        
def insert(curve, frame, value, no_tresh=False):
    bstart = [fc for fc in curve.data.animation_data.action.fcurves if fc.data_path == "bevel_factor_start"][0]
    bend   = [fc for fc in curve.data.animation_data.action.fcurves if fc.data_path == "bevel_factor_end"][0]
    f = bend.keyframe_points.insert(frame+1, value)
    f.co[0] -= 1
    tresh = 0 if no_tresh else frameTRESHOLD
    f = bstart.keyframe_points.insert(frame+tresh+1,value)
    f.co[0] -= 1
    
def linearize_fcurves(curve):
    bstart = [fc for fc in curve.data.animation_data.action.fcurves if fc.data_path == "bevel_factor_start"][0]
    bend   = [fc for fc in curve.data.animation_data.action.fcurves if fc.data_path == "bevel_factor_end"][0]
    for kf in bend.keyframe_points: kf.interpolation = "LINEAR"
    for kf in bstart.keyframe_points: kf.interpolation = "LINEAR"    

def mapToFrame(real):
    frame = (int(real)-realSTART)/(realEND-realSTART)*frameDURATION
    return floor(frame)

def do3group(group):
    routeName = group[0][0]
    fromName = group[0][3]
    toName = group[2][3]
    
    if fromName == "NORH":
        return
    
    route1 = bpy.data.objects[routeName+"_"+fromName+"_start"]
    route2 = bpy.data.objects[routeName+"_"+toName+"_end"]

    insert(route1, mapToFrame(group[0][5]), 0)
    insert(route1, mapToFrame(group[1][4]), 1)
    insert(route1, mapToFrame(group[1][4])+frameTRESHOLD, 1, no_tresh=True)
    insert(route1, mapToFrame(group[1][4])+frameTRESHOLD, 0, no_tresh=True)
    insert(route2, mapToFrame(group[1][5]), 0)
    insert(route2, mapToFrame(group[2][4]), 1)
    insert(route2, mapToFrame(group[2][4])+frameTRESHOLD, 1, no_tresh=True)
    insert(route2, mapToFrame(group[2][4])+frameTRESHOLD, 0, no_tresh=True)
    
def do2group(group):
    routeName = group[0][0]
    toName = group[1][3]
    if routeName == "40" and toName == "RADI":
        return
    route = bpy.data.objects[routeName+"_"+toName+"_end"]
    
    insert(route, mapToFrame(group[0][5]), 0)
    insert(route, mapToFrame(group[1][4]), 1)
    insert(route, mapToFrame(group[1][4])+frameTRESHOLD, 1, no_tresh=True)
    insert(route, mapToFrame(group[1][4])+frameTRESHOLD, 0, no_tresh=True)
    

s = "/Users/treyerl/Documents/iA/Teaching/HS13/data/public-transportation/zurich/schedule_5-10am.csv"
f = open(s, 'rt', encoding="utf8") 
timecsv = csv.reader(f, delimiter=",")

curves = [c for c in bpy.data.objects if c.type == "CURVE" and c.layers[0]]

for c in curves:
    animate(c)
    if bpy.data.objects.find(c.name+"_bevel") == -1:
        bpy.ops.curve.primitive_bezier_circle_add()
        last = bpy.context.active_object
        last.name = c.name+"_bevel"
        last.parent = bpy.data.objects['Bevels']
        last.layers[4] = True
        last.layers[0] = False
    c.data.bevel_object = bpy.data.objects[c.name+"_bevel"]
    c.data.taper_object  = bpy.data.objects['Taper']
    c.data.use_map_taper = True

for i,line in enumerate(timecsv):
    if i == 0: continue
    if i == 1:
        first = line
        group = [first]
        continue
    if int(line[5]) < realSTART or int(line[4]) > realEND:
        continue
    
    #  line[1] = tripId
    if line[1] != first[1]: 
        if len(group) == 3:
            do3group(group)
        if len(group) == 2:
            do2group(group)
        first = line
        group = [first]
    else:
        group.append(line)

for c in curves:
    linearize_fcurves(c)